@@ -78,25 +78,30 @@ where
                     match qualified_rule {
                         Ok(i) => rules.push(Rule::QualifiedRule(i)),
                         Err(err) => {
-                            if is!(self, "}") {
-                                self.errors.push(err);
-                                self.input.reset(&state);
+                            self.errors.push(err);
+                            self.input.reset(&state);
 
-                                let start_pos = self.input.cur_span()?.lo;
+                            let span = self.input.cur_span()?;
+                            let mut tokens = vec![];
 
-                                let mut tokens = vec![];
+                            while !is_one_of!(self, EOF, "}") {
+                                let token = self.input.bump()?;
 
-                                while !is_one_of!(self, EOF, "}") {
-                                    tokens.extend(self.input.bump()?);
+                                tokens.extend(token);
+
+                                if is!(self, ";") {
+                                    let token = self.input.bump()?;
+
+                                    tokens.extend(token);
+
+                                    break;
                                 }
-
-                                rules.push(Rule::Invalid(Tokens {
-                                    span: span!(self, start_pos),
-                                    tokens,
-                                }));
-                            } else {
-                                return Err(err);
                             }
+
+                            rules.push(Rule::Invalid(Tokens {
+                                span: span!(self, span.lo),
+                                tokens,
+                            }));
                         }
                     };
                 }
@@ -111,15 +116,18 @@ where
 {
     fn parse(&mut self) -> PResult<QualifiedRule> {
         let span = self.input.cur_span()?;
-        let mut prelude = SelectorList {
+        // Create a new qualified rule with its prelude initially set to an empty list,
+        // and its value initially set to nothing.
+        let mut prelude = QualifiedRulePrelude::Invalid(Tokens {
             span: Default::default(),
-            children: vec![],
-        };
+            tokens: vec![],
+        });
 
         // Repeatedly consume the next input token:
         loop {
             // <EOF-token>
-            // Return the list of rules.
+            // This is a parse error. Return nothing.
+            // But we return for error recovery blocks
             if is!(self, EOF) {
                 let span = self.input.cur_span()?;
 
@@ -146,7 +154,35 @@ where
                 // Reconsume the current input token. Consume a component value. Append the returned
                 // value to the qualified rule’s prelude.
                 _ => {
-                    prelude = self.parse()?;
+                    let state = self.input.state();
+
+                    prelude = match self.parse() {
+                        Ok(selector_list) => QualifiedRulePrelude::SelectorList(selector_list),
+                        Err(err) => {
+                            self.errors.push(err);
+                            self.input.reset(&state);
+
+                            let span = self.input.cur_span()?;
+                            let mut tokens = vec![];
+
+                            while !is_one_of!(self, EOF, "{") {
+                                if is!(self, ";") {
+                                    let span = self.input.cur_span()?;
+
+                                    return Err(Error::new(span, ErrorKind::UnexpectedChar(';')));
+                                }
+
+                                let token = self.input.bump()?;
+
+                                tokens.extend(token);
+                            }
+
+                            QualifiedRulePrelude::Invalid(Tokens {
+                                span: span!(self, span.lo),
+                                tokens,
+                            })
+                        }
+                    };
                 }
             }
         }
@@ -233,17 +269,27 @@ where
                 // returned, append it to rules.
                 tok!("&") => {
                     let state = self.input.state();
-                    let span = self.input.cur_span()?;
                     let qualified_rule = match self.parse() {
                         Ok(v) => StyleBlock::QualifiedRule(v),
                         Err(err) => {
                             self.errors.push(err);
                             self.input.reset(&state);
 
+                            let span = self.input.cur_span()?;
                             let mut tokens = vec![];
 
                             while !is_one_of!(self, EOF, "}") {
-                                tokens.extend(self.input.bump()?);
+                                let token = self.input.bump()?;
+
+                                tokens.extend(token);
+
+                                if is!(self, ";") {
+                                    let token = self.input.bump()?;
+
+                                    tokens.extend(token);
+
+                                    break;
+                                }
                             }
 
                             StyleBlock::Invalid(Tokens {
@@ -327,7 +373,10 @@ where
                 '['
             }
             _ => {
-                unreachable!();
+                return Err(Error::new(
+                    span,
+                    ErrorKind::Expected("'{', '(' or '[' token"),
+                ));
             }
         };
         // Create a simple block with its associated token set to the current input
